/**************************************
 *   Programm: Mandelbrot Demo        *
 *   Info:     Beispiel fr Artikel   *
 *             in der A&M Info        *
 *   Autor:    Hanns Holger Rutz      *
 *             (PublicDomain)         *
 *   Datum:    22.10.1993             *
 *   Sprache:  GNU-C (mit libauto.a   *
 *             linken)                *
 **************************************/

#include <exec/types.h>
#include <intuition/intuition.h>
#include <intuition/screens.h>
#include <math.h>
#include <signal.h>

#define PixWidth   (260)                   /* Breite der Graphik in Pixeln */
#define PixHeight  (120)                   /* Hhe der Graphik in Zeilen */
#define RealStart  (-2)                    /* untere Grenze der Realwerte */
#define RealEnd    (0.6)                   /* obere Grenze der Realwerte */
#define ImagiStart (-1.3)                  /* untere Grenze der Imaginrwerte */
#define ImagiEnd   (1.3)                   /* obere Grenze der Imaginrwerte */
#define RealWidth  (RealEnd-RealStart)
#define ImagiWidth (ImagiEnd-ImagiStart)
#define RealStep   (RealWidth/PixWidth)    /* Schritte auf Realachse pro Pixel */
#define ImagiStep  (ImagiWidth/PixHeight)  /* Schritte auf Imaginrachse pro Zeile */
#define IterMax    (100)                   /* maximale Anzahl von Interationen */
#define BetragMax  (2)                     /* Schwellwert fr Entfernung des Punktes */

#define Txt1       "PublicDomain von Holgi"
#define Txt2       "(CloseGadget = Ende)"

struct Library *IntuitionBase, *GfxBase;
struct Window *MyWin;
struct Screen *MyScr;
struct RastPort *MyRast;
struct DrawInfo *DrInfo;
struct TextFont *MyFont;

struct TagItem WindowTags[] =
{
   WA_InnerWidth, PixWidth,
   WA_InnerHeight, PixHeight+22,
   WA_Flags, WFLG_DRAGBAR | WFLG_RMBTRAP | WFLG_SMART_REFRESH | WFLG_CLOSEGADGET |
             WFLG_DEPTHGADGET,
   WA_PubScreenName, NULL,
   WA_IDCMP, IDCMP_CLOSEWINDOW,
   WA_Title, (ULONG) "Mandelbrot Demo",
   TAG_DONE, 0
};

struct TextAttr TopazAttr =
{
   "topaz.font",
   8,
   FS_NORMAL,FPF_ROMFONT | FPF_DESIGNED
};

float ZReal, ZImagi,                       /* springender Real- und Imaginrwert */
      CReal, CImagi,                       /* Ausgangs- Real- und Imaginrwert */
      Betrag,                              /* Entfernung von ZReal+ZImagi zum Nullpunkt */
      Foo;                                 /* zur Zwischenspeicherung */
UWORD XStart, YStart,                      /* linke, obere Ecke des RastPorts */
      XPos, YPos,                          /* aktuelle RastPort Koordinaten */
      Iteration,                           /* Iterationszhler */
      InCol = 1, OutCol = 2, TxtCol = 1;   /* verwendete Farben */
BOOL  LastIn = FALSE;                      /* zuletzt benutze Farbe (um SetAPen-
                                              Aufrufe zu sparen) */
ULONG SigMask;                             /* zum Warten auf CloseGadget */

/**********************************
 Hauptprogramm               
 **********************************/

main()
{
#ifdef __GNUC__
   signal( SIGINT, SIG_IGN );              /* CTRL+C Breakhandling beim GNU-C aus */
#endif

   if( MyWin = OpenWindowTagList( NULL, &WindowTags ))
   {
      MyRast  = MyWin->RPort;
      MyScr   = MyWin->WScreen;
      XStart  = MyScr->WBorLeft;
      YStart  = MyScr->BarHeight + 1;
      SigMask = 1<<(MyWin->UserPort->mp_SigBit);

      if( DrInfo = GetScreenDrawInfo( MyScr ))
      {
         InCol  = DrInfo->dri_Pens[ SHADOWPEN ];
         OutCol = DrInfo->dri_Pens[ SHINEPEN ];
         TxtCol = DrInfo->dri_Pens[ TEXTPEN ];
         FreeScreenDrawInfo( MyScr, DrInfo );
      }

      if( MyFont = OpenFont( &TopazAttr ))
      {
         SetFont( MyRast, MyFont );
         SetAPen( MyRast, TxtCol );
         Move( MyRast, XStart + (PixWidth - (sizeof( Txt1 ) - 1) * 8) / 2,
                       YStart + PixHeight + 8 );
         Text( MyRast, Txt1, sizeof( Txt1 )-1 );
         Move( MyRast, XStart + (PixWidth - (sizeof( Txt2 ) - 1) * 8) / 2,
                       YStart + PixHeight + 18 );
         Text( MyRast, Txt2, sizeof( Txt2 )-1 );
      }

      SetAPen( MyRast, OutCol );       /* voreingestellte Farbe (auerhalb der Menge) */
      CImagi = ImagiStart;                        /* CImagi initialisieren */
      for( YPos = 0; YPos < PixHeight; YPos++ )   /* Zeilenschleife */
      {
         CReal = RealStart;                       /* CReal initialisieren */
         for( XPos = 0; XPos < PixWidth; XPos++ ) /* Spaltenschleife */
         {
            Iteration = 0;                        /* mit Iteration beginnen */
            ZReal     = 0;                        /* Sprungwerte */
            ZImagi    = 0;                        /* ...initialisieren */
            do
            {
               Foo    = ZReal;
               /* neuer Realwert =  (alter Real)^2 - (alter Imagi)^2 + CReal */
               ZReal  = ZReal * ZReal - ZImagi * ZImagi + CReal;
               /* neuer Imagiwert = (2 * alter Real * alter Imagi)   + CImagi */
               ZImagi = 2 * Foo * ZImagi + CImagi;
               /* Entfernung nach Pythagoras berechnen */
               Betrag = (float) sqrt( (double) (ZReal * ZReal + ZImagi * ZImagi) );
               /* Iteration erhhen */
               Iteration++;
            }           
            while( (Iteration < IterMax) & (Betrag <= BetragMax) );
            /* (Schleife fortsetzen bis max. Iteration oder Grenzwert erreicht) */

            if( Iteration < IterMax )           /* wir sind auerhalb der Menge */
            {
               if( LastIn )
               {
                  LastIn = FALSE;
                  SetAPen( MyRast, OutCol );    /* Farbe wechseln */
               }
            }
            else                                /* wir sind im Mandelbrot */
            {
               if( !LastIn )
               {
                  LastIn = TRUE;
                  SetAPen( MyRast, InCol );     /* Farbe wechseln */
               }
            }             
            WritePixel( MyRast, XPos+XStart, YPos+YStart );  /* Pixel einfrben */

            CReal += RealStep;                  /* auf der Realachse nach rechts */
         }

         if( CheckSignal( SigMask ))
            break;                              /* (CloseGadget bettigt) */

         CImagi += ImagiStep;                   /* auf der Imaginrachse nach unten */
      }

      WaitPort( MyWin->UserPort );              /* auf CloseGadget warten */
      CloseWindow( MyWin );
      if( MyFont ) CloseFont( MyFont );
   }
}
